package java.awt.image;

import java.awt.Rectangle;
import java.awt.Point;
import java.util.Set;
import java.util.Vector;
import java.awt.GraphicsEnvironment;
import java.awt.Graphics2D;
import java.awt.Graphics;
import java.security.AccessController;
import java.security.PrivilegedAction;
import java.util.Iterator;
import sun.awt.image.BytePackedRaster;
import sun.awt.image.ByteComponentRaster;
import sun.awt.image.ShortComponentRaster;
import sun.awt.image.IntegerComponentRaster;
import java.awt.color.ColorSpace;
import java.util.Hashtable;
import sun.awt.image.OffScreenImageSource;
import java.awt.Transparency;
import java.awt.Image;

public class BufferedImage extends Image implements WritableRenderedImage, Transparency
{
    private int imageType;
    private ColorModel colorModel;
    private final WritableRaster raster;
    private OffScreenImageSource osis;
    private Hashtable<String, Object> properties;
    public static final int TYPE_CUSTOM = 0;
    public static final int TYPE_INT_RGB = 1;
    public static final int TYPE_INT_ARGB = 2;
    public static final int TYPE_INT_ARGB_PRE = 3;
    public static final int TYPE_INT_BGR = 4;
    public static final int TYPE_3BYTE_BGR = 5;
    public static final int TYPE_4BYTE_ABGR = 6;
    public static final int TYPE_4BYTE_ABGR_PRE = 7;
    public static final int TYPE_USHORT_565_RGB = 8;
    public static final int TYPE_USHORT_555_RGB = 9;
    public static final int TYPE_BYTE_GRAY = 10;
    public static final int TYPE_USHORT_GRAY = 11;
    public static final int TYPE_BYTE_BINARY = 12;
    public static final int TYPE_BYTE_INDEXED = 13;
    private static final int DCM_RED_MASK = 16711680;
    private static final int DCM_GREEN_MASK = 65280;
    private static final int DCM_BLUE_MASK = 255;
    private static final int DCM_ALPHA_MASK = -16777216;
    private static final int DCM_565_RED_MASK = 63488;
    private static final int DCM_565_GRN_MASK = 2016;
    private static final int DCM_565_BLU_MASK = 31;
    private static final int DCM_555_RED_MASK = 31744;
    private static final int DCM_555_GRN_MASK = 992;
    private static final int DCM_555_BLU_MASK = 31;
    private static final int DCM_BGR_RED_MASK = 255;
    private static final int DCM_BGR_GRN_MASK = 65280;
    private static final int DCM_BGR_BLU_MASK = 16711680;
    
    private static native void initIDs();
    
    public BufferedImage(int n, int n2, int n3) {
        this.imageType = 0;
        switch (n3) {
            case 1: {
                this.colorModel = new DirectColorModel(24, 16711680, 65280, 255, 0);
                this.raster = this.colorModel.createCompatibleWritableRaster(n, n2);
                break;
            }
            case 2: {
                this.colorModel = ColorModel.getRGBdefault();
                this.raster = this.colorModel.createCompatibleWritableRaster(n, n2);
                break;
            }
            case 3: {
                this.colorModel = new DirectColorModel(ColorSpace.getInstance(1000), 32, 16711680, 65280, 255, -16777216, true, 3);
                this.raster = this.colorModel.createCompatibleWritableRaster(n, n2);
                break;
            }
            case 4: {
                this.colorModel = new DirectColorModel(24, 255, 65280, 16711680);
                this.raster = this.colorModel.createCompatibleWritableRaster(n, n2);
                break;
            }
            case 5: {
                ColorSpace instance = ColorSpace.getInstance(1000);
                int[] array = { 8, 8, 8 };
                int[] array2 = { 2, 1, 0 };
                this.colorModel = new ComponentColorModel(instance, array, false, false, 1, 0);
                this.raster = Raster.createInterleavedRaster(0, n, n2, n * 3, 3, array2, null);
                break;
            }
            case 6: {
                ColorSpace instance2 = ColorSpace.getInstance(1000);
                int[] array3 = { 8, 8, 8, 8 };
                int[] array4 = { 3, 2, 1, 0 };
                this.colorModel = new ComponentColorModel(instance2, array3, true, false, 3, 0);
                this.raster = Raster.createInterleavedRaster(0, n, n2, n * 4, 4, array4, null);
                break;
            }
            case 7: {
                ColorSpace instance3 = ColorSpace.getInstance(1000);
                int[] array5 = { 8, 8, 8, 8 };
                int[] array6 = { 3, 2, 1, 0 };
                this.colorModel = new ComponentColorModel(instance3, array5, true, true, 3, 0);
                this.raster = Raster.createInterleavedRaster(0, n, n2, n * 4, 4, array6, null);
                break;
            }
            case 10: {
                this.colorModel = new ComponentColorModel(ColorSpace.getInstance(1003), new int[] { 8 }, false, true, 1, 0);
                this.raster = this.colorModel.createCompatibleWritableRaster(n, n2);
                break;
            }
            case 11: {
                this.colorModel = new ComponentColorModel(ColorSpace.getInstance(1003), new int[] { 16 }, false, true, 1, 1);
                this.raster = this.colorModel.createCompatibleWritableRaster(n, n2);
                break;
            }
            case 12: {
                byte[] array7 = { 0, -1 };
                this.colorModel = new IndexColorModel(1, 2, array7, array7, array7);
                this.raster = Raster.createPackedRaster(0, n, n2, 1, 1, null);
                break;
            }
            case 13: {
                int[] array8 = new int[256];
                int i = 0;
                for (int j = 0; j < 256; j += 51) {
                    for (int k = 0; k < 256; k += 51) {
                        for (int l = 0; l < 256; l += 51) {
                            array8[i++] = (j << 16 | k << 8 | l);
                        }
                    }
                }
                int n4 = 256 / (256 - i);
                int n5 = n4 * 3;
                while (i < 256) {
                    array8[i] = (n5 << 16 | n5 << 8 | n5);
                    n5 += n4;
                    ++i;
                }
                this.colorModel = new IndexColorModel(8, 256, array8, 0, false, -1, 0);
                this.raster = Raster.createInterleavedRaster(0, n, n2, 1, null);
                break;
            }
            case 8: {
                this.colorModel = new DirectColorModel(16, 63488, 2016, 31);
                this.raster = this.colorModel.createCompatibleWritableRaster(n, n2);
                break;
            }
            case 9: {
                this.colorModel = new DirectColorModel(15, 31744, 992, 31);
                this.raster = this.colorModel.createCompatibleWritableRaster(n, n2);
                break;
            }
            default: {
                throw new IllegalArgumentException("Unknown image type " + n3);
            }
        }
        this.imageType = n3;
    }
    
    public BufferedImage(int n, int n2, int n3, IndexColorModel colorModel) {
        this.imageType = 0;
        if (colorModel.hasAlpha() && colorModel.isAlphaPremultiplied()) {
            throw new IllegalArgumentException("This image types do not have premultiplied alpha.");
        }
        switch (n3) {
            case 12: {
                int mapSize = colorModel.getMapSize();
                int n4;
                if (mapSize <= 2) {
                    n4 = 1;
                }
                else if (mapSize <= 4) {
                    n4 = 2;
                }
                else if (mapSize <= 16) {
                    n4 = 4;
                }
                else {
                    throw new IllegalArgumentException("Color map for TYPE_BYTE_BINARY must have no more than 16 entries");
                }
                this.raster = Raster.createPackedRaster(0, n, n2, 1, n4, null);
                break;
            }
            case 13: {
                this.raster = Raster.createInterleavedRaster(0, n, n2, 1, null);
                break;
            }
            default: {
                throw new IllegalArgumentException("Invalid image type (" + n3 + ").  Image type must be either TYPE_BYTE_BINARY or  TYPE_BYTE_INDEXED");
            }
        }
        if (!colorModel.isCompatibleRaster(this.raster)) {
            throw new IllegalArgumentException("Incompatible image type and IndexColorModel");
        }
        this.colorModel = colorModel;
        this.imageType = n3;
        return;
    }
    
    public BufferedImage(ColorModel colorModel, WritableRaster obj, boolean b, Hashtable<?, ?> hashtable) {
        this.imageType = 0;
        if (!colorModel.isCompatibleRaster(obj)) {
            throw new IllegalArgumentException("Raster " + obj + " is incompatible with ColorModel " + colorModel);
        }
        if (obj.minX != 0 || obj.minY != 0) {
            throw new IllegalArgumentException("Raster " + obj + " has minX or minY not equal to zero: " + obj.minX + " " + obj.minY);
        }
        this.colorModel = colorModel;
        this.raster = obj;
        if (hashtable != null && !hashtable.isEmpty()) {
            this.properties = new Hashtable();
            for (Object next : hashtable.keySet()) {
                if ((next instanceof String)) {
                    this.properties.put((String)next, hashtable.get(next));
                }
                else {
                    continue;
                }
            }
        }
        int numBands = obj.getNumBands();
        int alphaPremultiplied = colorModel.isAlphaPremultiplied();
        int standard = isStandard(colorModel, obj);
        this.coerceData(b);
        SampleModel sampleModel = obj.getSampleModel();
        int type = colorModel.getColorSpace().getType();
        if (type != 5) {
            if (type == 6 && standard && colorModel instanceof ComponentColorModel) {
                if (sampleModel instanceof ComponentSampleModel && ((ComponentSampleModel)sampleModel).getPixelStride() != numBands) {
                    this.imageType = 0;
                }
                else if (obj instanceof ByteComponentRaster && obj.getNumBands() == 1 && colorModel.getComponentSize(0) == 8 && ((ByteComponentRaster)obj).getPixelStride() == 1) {
                    this.imageType = 10;
                }
                else if (obj instanceof ShortComponentRaster && obj.getNumBands() == 1 && colorModel.getComponentSize(0) == 16 && ((ShortComponentRaster)obj).getPixelStride() == 1) {
                    this.imageType = 11;
                }
            }
            else {
                this.imageType = 0;
            }
            return;
        }
        else {
            if (obj instanceof IntegerComponentRaster && (numBands == 3 || numBands == 4)) {
                IntegerComponentRaster integerComponentRaster = (IntegerComponentRaster)obj;
                int pixelSize2 = colorModel.getPixelSize();
                if (integerComponentRaster.getPixelStride() == 1 && standard && colorModel instanceof DirectColorModel && (pixelSize2 == 32 || pixelSize2 == 24)) {
                    DirectColorModel directColorModel2 = (DirectColorModel)colorModel;
                    int redMask = directColorModel2.getRedMask();
                    int greenMask = directColorModel2.getGreenMask();
                    int blueMask = directColorModel2.getBlueMask();
                    if (redMask == 16711680 && greenMask == 65280 && blueMask == 255) {
                        if (directColorModel2.getAlphaMask() == -16777216) {
                            this.imageType = (alphaPremultiplied ? 3 : 2);
                        }
                        else if (!directColorModel2.hasAlpha()) {
                            this.imageType = 1;
                        }
                    }
                    else if (redMask == 255 && greenMask == 65280 && blueMask == 16711680 && !directColorModel2.hasAlpha()) {
                        this.imageType = 4;
                    }
                }
            }
            else if (colorModel instanceof IndexColorModel && numBands == 1 && standard && (!colorModel.hasAlpha() || alphaPremultiplied == 0)) {
                int pixelSize = ((ColorModel)colorModel).getPixelSize();
                if ((obj instanceof BytePackedRaster)) {
                    this.imageType = 12;
                }
                else if (obj instanceof ByteComponentRaster && ((ByteComponentRaster)obj).getPixelStride() == 1 && pixelSize <= 8) {
                    this.imageType = 13;
                }
            }
            else if (obj instanceof ShortComponentRaster && colorModel instanceof DirectColorModel && standard && numBands == 3 && !colorModel.hasAlpha()) {
                DirectColorModel directColorModel = (DirectColorModel)colorModel;
                if (directColorModel.getRedMask() == 63488) {
                    if (directColorModel.getGreenMask() == 2016 && directColorModel.getBlueMask() == 31) {
                        this.imageType = 8;
                    }
                }
                else if (directColorModel.getRedMask() == 31744 && directColorModel.getGreenMask() == 992 && directColorModel.getBlueMask() == 31) {
                    this.imageType = 9;
                }
            }
            else if (obj instanceof ByteComponentRaster && colorModel instanceof ComponentColorModel && standard && obj.getSampleModel() instanceof PixelInterleavedSampleModel && (numBands == 3 || numBands == 4)) {
                ComponentColorModel componentColorModel = (ComponentColorModel)colorModel;
                PixelInterleavedSampleModel pixelInterleavedSampleModel = (PixelInterleavedSampleModel)obj.getSampleModel();
                ByteComponentRaster byteComponentRaster = (ByteComponentRaster)obj;
                int[] bandOffsets = pixelInterleavedSampleModel.getBandOffsets();
                if (componentColorModel.getNumComponents() != numBands) {
                    throw new RasterFormatException("Number of components in ColorModel (" + componentColorModel.getNumComponents() + ") does not match # in  Raster (" + numBands + ")");
                }
                int[] componentSize = componentColorModel.getComponentSize();
                boolean n = true;
                int i = 0;
                while (i < numBands) {
                    if (componentSize[i] != 8) {
                        n = false;
                        break;
                    }
                    ++i;
                }
                if (n && byteComponentRaster.getPixelStride() == numBands && bandOffsets[0] == numBands - 1 && bandOffsets[1] == numBands - 2 && bandOffsets[2] == numBands - 3) {
                    if (numBands == 3 && !componentColorModel.hasAlpha()) {
                        this.imageType = 5;
                    }
                    else if (bandOffsets[3] == 0 && componentColorModel.hasAlpha()) {
                        this.imageType = (alphaPremultiplied ? 7 : 6);
                    }
                }
            }
            return;
        }
    }
    
    private static boolean isStandard(ColorModel colorModel, WritableRaster writableRaster) {
        return (boolean)AccessController.doPrivileged(new PrivilegedAction<Boolean>() {
            
            @Override
            public Boolean run() {
                ClassLoader classLoader = System.class.getClassLoader();
                return Boolean.valueOf(cmClass.getClassLoader() == classLoader && smClass.getClassLoader() == classLoader && wrClass.getClassLoader() == classLoader);
            }
        });
    }
    
    public int getType() {
        return this.imageType;
    }
    
    @Override
    public ColorModel getColorModel() {
        return this.colorModel;
    }
    
    public WritableRaster getRaster() {
        return this.raster;
    }
    
    public WritableRaster getAlphaRaster() {
        return this.colorModel.getAlphaRaster(this.raster);
    }
    
    public int getRGB(int n, int n2) {
        return this.colorModel.getRGB(this.raster.getDataElements(n, n2, null));
    }
    
    public int[] getRGB(int i, int j, int n, int n2, int[] array, int n3, int n4) {
        int numBands = this.raster.getNumBands();
        int dataType = this.raster.getDataBuffer().getDataType();
        byte[] array2;
        switch (dataType) {
            case 0: {
                array2 = new byte[numBands];
                break;
            }
            case 1: {
                array2 = new short[numBands];
                break;
            }
            case 3: {
                array2 = new int[numBands];
                break;
            }
            case 4: {
                array2 = new float[numBands];
                break;
            }
            case 5: {
                array2 = new double[numBands];
                break;
            }
            default: {
                throw new IllegalArgumentException("Unknown data buffer type: " + dataType);
            }
        }
        if (array == null) {
            array = new int[n3 + n2 * n4];
        }
        for (; j < j + n2; ++j, n3 += n4) {
            for (; i < i + n; ++i) {
                array[n3++] = this.colorModel.getRGB(this.raster.getDataElements(i, j, array2));
            }
        }
        return array;
    }
    
    public synchronized void setRGB(int n, int n2, int n3) {
        this.raster.setDataElements(n, n2, this.colorModel.getDataElements(n3, null));
    }
    
    public void setRGB(int i, int j, int n, int n2, int[] array, int n3, int n4) {
        Object dataElements = null;
        for (; j < j + n2; ++j, n3 += n4) {
            for (; i < i + n; ++i) {
                dataElements = this.colorModel.getDataElements(array[n3++], dataElements);
                this.raster.setDataElements(i, j, dataElements);
            }
        }
    }
    
    @Override
    public int getWidth() {
        return this.raster.getWidth();
    }
    
    @Override
    public int getHeight() {
        return this.raster.getHeight();
    }
    
    @Override
    public int getWidth(ImageObserver imageObserver) {
        return this.raster.getWidth();
    }
    
    @Override
    public int getHeight(ImageObserver imageObserver) {
        return this.raster.getHeight();
    }
    
    @Override
    public ImageProducer getSource() {
        if (this.osis == null) {
            if (this.properties == null) {
                this.properties = new Hashtable();
            }
            this.osis = new OffScreenImageSource(this, this.properties);
        }
        return this.osis;
    }
    
    @Override
    public Object getProperty(String s, ImageObserver imageObserver) {
        return this.getProperty(s);
    }
    
    @Override
    public Object getProperty(String key) {
        if (key == null) {
            throw new NullPointerException("null property name is not allowed");
        }
        if (this.properties == null) {
            return Image.UndefinedProperty;
        }
        Object o = this.properties.get(key);
        if (o == null) {
            o = Image.UndefinedProperty;
            return o;
        }
        return o;
    }
    
    @Override
    public Graphics getGraphics() {
        return this.createGraphics();
    }
    
    public Graphics2D createGraphics() {
        return GraphicsEnvironment.getLocalGraphicsEnvironment().createGraphics(this);
    }
    
    public BufferedImage getSubimage(int n, int n2, int n3, int n4) {
        return new BufferedImage(this.colorModel, this.raster.createWritableChild(n, n2, n3, n4, 0, 0, (int[])null), this.colorModel.isAlphaPremultiplied(), this.properties);
    }
    
    public boolean isAlphaPremultiplied() {
        return this.colorModel.isAlphaPremultiplied();
    }
    
    public void coerceData(boolean b) {
        if (this.colorModel.hasAlpha() && this.colorModel.isAlphaPremultiplied() != b) {
            this.colorModel = this.colorModel.coerceData(this.raster, b);
        }
    }
    
    @Override
    public String toString() {
        return "BufferedImage@" + Integer.toHexString(this.hashCode()) + ": type = " + this.imageType + " " + this.colorModel + " " + this.raster;
    }
    
    @Override
    public Vector<RenderedImage> getSources() {
        return null;
    }
    
    @Override
    public String[] getPropertyNames() {
        if (this.properties == null || this.properties.isEmpty()) {
            return null;
        }
        Set<String> keySet = this.properties.keySet();
        return (String[])keySet.toArray(new String[keySet.size()]);
    }
    
    @Override
    public int getMinX() {
        return this.raster.getMinX();
    }
    
    @Override
    public int getMinY() {
        return this.raster.getMinY();
    }
    
    @Override
    public SampleModel getSampleModel() {
        return this.raster.getSampleModel();
    }
    
    @Override
    public int getNumXTiles() {
        return 1;
    }
    
    @Override
    public int getNumYTiles() {
        return 1;
    }
    
    @Override
    public int getMinTileX() {
        return 0;
    }
    
    @Override
    public int getMinTileY() {
        return 0;
    }
    
    @Override
    public int getTileWidth() {
        return this.raster.getWidth();
    }
    
    @Override
    public int getTileHeight() {
        return this.raster.getHeight();
    }
    
    @Override
    public int getTileGridXOffset() {
        return this.raster.getSampleModelTranslateX();
    }
    
    @Override
    public int getTileGridYOffset() {
        return this.raster.getSampleModelTranslateY();
    }
    
    @Override
    public Raster getTile(int n, int n2) {
        if (n == 0 && n2 == 0) {
            return this.raster;
        }
        throw new ArrayIndexOutOfBoundsException("BufferedImages only have one tile with index 0,0");
    }
    
    @Override
    public Raster getData() {
        int width = this.raster.getWidth();
        int height = this.raster.getHeight();
        int minX = this.raster.getMinX();
        int i = this.raster.getMinY();
        WritableRaster writableRaster = Raster.createWritableRaster(this.raster.getSampleModel(), new Point(this.raster.getSampleModelTranslateX(), this.raster.getSampleModelTranslateY()));
        Object dataElements = null;
        for (; i < i + height; ++i) {
            dataElements = this.raster.getDataElements(minX, i, width, 1, dataElements);
            writableRaster.setDataElements(minX, i, width, 1, dataElements);
        }
        return writableRaster;
    }
    
    @Override
    public Raster getData(Rectangle rectangle) {
        WritableRaster writableRaster = Raster.createWritableRaster(this.raster.getSampleModel().createCompatibleSampleModel(rectangle.width, rectangle.height), rectangle.getLocation());
        int width = rectangle.width;
        int height = rectangle.height;
        int x = rectangle.x;
        int i = rectangle.y;
        Object dataElements = null;
        for (; i < i + height; ++i) {
            dataElements = this.raster.getDataElements(x, i, width, 1, dataElements);
            writableRaster.setDataElements(x, i, width, 1, dataElements);
        }
        return writableRaster;
    }
    
    @Override
    public WritableRaster copyData(WritableRaster writableRaster) {
        if (writableRaster == null) {
            return (WritableRaster)this.getData();
        }
        int width = writableRaster.getWidth();
        int height = writableRaster.getHeight();
        int minX = writableRaster.getMinX();
        int i = writableRaster.getMinY();
        Object dataElements = null;
        for (; i < i + height; ++i) {
            dataElements = this.raster.getDataElements(minX, i, width, 1, dataElements);
            writableRaster.setDataElements(minX, i, width, 1, dataElements);
        }
        return writableRaster;
    }
    
    @Override
    public void setData(Raster raster) {
        int width = raster.getWidth();
        int height = raster.getHeight();
        int minX = raster.getMinX();
        int minY = raster.getMinY();
        int[] pixels = null;
        Rectangle intersection = new Rectangle(minX, minY, width, height).intersection(new Rectangle(0, 0, this.raster.width, this.raster.height));
        if (intersection.isEmpty()) {
            return;
        }
        int width2 = intersection.width;
        int height2 = intersection.height;
        int x = intersection.x;
        for (int i = intersection.y; i < i + height2; ++i) {
            pixels = raster.getPixels(x, i, width2, 1, pixels);
            this.raster.setPixels(x, i, width2, 1, pixels);
        }
        return;
    }
    
    @Override
    public void addTileObserver(TileObserver tileObserver) {
    }
    
    @Override
    public void removeTileObserver(TileObserver tileObserver) {
    }
    
    @Override
    public boolean isTileWritable(int n, int n2) {
        if (n == 0 && n2 == 0) {
            return true;
        }
        throw new IllegalArgumentException("Only 1 tile in image");
    }
    
    @Override
    public Point[] getWritableTileIndices() {
        return new Point[] { new Point(0, 0) };
    }
    
    @Override
    public boolean hasTileWriters() {
        return true;
    }
    
    @Override
    public WritableRaster getWritableTile(int n, int n2) {
        return this.raster;
    }
    
    @Override
    public void releaseWritableTile(int n, int n2) {
    }
    
    @Override
    public int getTransparency() {
        return this.colorModel.getTransparency();
    }
    
    static {
        ColorModel.loadLibraries();
        initIDs();
    }
}
